@@ -147,10 +147,17 @@ _CREATE_ACTION = "Create"
 _UPDATE_ACTION = "Update"
 _DELETE_ACTION = "Delete"
 _VIEW_ACTION = "View"
+_START_ACTION = "Start"
+_STOP_ACTION = "Stop"
 _START_TRAINING_ACTION = "StartTraining"
 _STOP_TRAINING_ACTION = "StopTraining"
+_RESET_TRAINING_ACTION = "ResetTraining"
 _START_ASSESSMENT_ACTION = "StartAssessment"
 _STOP_ASSESSMENT_ACTION = "StopAssessment"
+_START_LOGGING_ACTION = "StartLogging"
+_STOP_LOGGING_ACTION = "StopLogging"
+_ASSESSMENT_OBJECT = "Assessment"
+_ALL_ASSESSMENTS_OBJECT = "Assessments"
 _BRAIN_OBJECT = "Brain"
 _ALL_EXPORTED_BRAINS_OBJECT = "ExportedBrains"
 _EXPORTED_BRAIN_OBJECT = "ExportedBrain"
@@ -480,7 +487,7 @@ class BonsaiAPI(object):
                 "timeout. Request ID: {}".format(url, self.timeout, req_id)
             )
 
-        response_dict = {}
+        response_dict: Dict[str, str] = {}
         try:
             response.raise_for_status()
             self._raise_on_redirect(response)
@@ -915,7 +922,7 @@ class BonsaiAPI(object):
 
     # callback function to monitor the progress
     def post_file_callback(self, monitor: MultipartEncoderMonitor):
-        log.info("NUMBER OF BYTES READY FOR UPLOAD: {}".format(monitor.bytes_read))
+        log.info("NUMBER OF BYTES UPLOADED: {}".format(monitor.bytes_read))
 
     # uploads model file in zip format
     def post_file(
@@ -951,8 +958,6 @@ class BonsaiAPI(object):
         output: Optional[str] = None,
     ):
 
-        log.information("****I am fine***")
-
         url_path = _CREATE_IMPORTED_MODEL_URL_PATH_TEMPLATE.format(
             workspaceid=workspace if workspace else self._workspace_id,
             importedmodelname=name,
@@ -1077,7 +1082,6 @@ class BonsaiAPI(object):
     def create_sim_package(
         self,
         name: str,
-        image_path: str,
         start_instance_count: int,
         min_instance_count: int,
         max_instance_count: int,
@@ -1085,6 +1089,9 @@ class BonsaiAPI(object):
         memory_in_gb_per_instance: float,
         auto_scale: bool,
         auto_terminate: bool,
+        image_path: Optional[str] = None,
+        model_file_path: Optional[str] = None,
+        model_base_image_name: Optional[str] = None,
         display_name: Optional[str] = None,
         description: Optional[str] = None,
         os_type: Optional[str] = None,
@@ -1114,6 +1121,8 @@ class BonsaiAPI(object):
             "osType": os_type,
             "packageType": package_type,
             "imagePath": image_path,
+            "modelFilePath": model_file_path,
+            "modelBaseImageName": model_base_image_name,
             "minInstanceCount": min_instance_count,
             "maxInstanceCount": max_instance_count,
             "autoScale": auto_scale,
@@ -1254,6 +1263,7 @@ class BonsaiAPI(object):
         auto_termination: Optional[str] = None,
         log_session_count: Optional[str] = None,
         include_system_logs: bool = False,
+        log_all_simulators: bool = False,
         workspace: Optional[str] = None,
         debug: bool = False,
         output: Optional[str] = None,
@@ -1281,16 +1291,14 @@ class BonsaiAPI(object):
         )
 
         simulatorLogConfig = json.loads(
-            '{{"sessionCount": "{}", "includeSystemLogs": "{}" }}'.format(
-                log_session_count, include_system_logs
+            '{{"sessionCount": "{}", "includeSystemLogs": "{}", "logAll": "{}" }}'.format(
+                log_session_count, include_system_logs, log_all_simulators
             )
         )
 
         data = {
             "purpose": purpose,
             "description": description,
-            "resourceGroupName": "",
-            "Subscription": "",
             "coresPerInstance": cores_per_instance,
             "memInGBPerInstance": memory_in_gb_per_instance,
             "startInstanceCount": start_instance_count,
@@ -1516,8 +1524,13 @@ class BonsaiAPI(object):
         }
 
         url = urljoin(self._api_url, url_path)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_START_LOGGING_ACTION, _BRAIN_VERSION_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_BRAIN_VERSION_OBJECT],
+        )
 
-        return self._post(url=url, data=data, debug=debug, output=output)
+        return self._post(url=url, data=data, debug=debug, output=output, event=event)
 
     def stop_logging(
         self,
@@ -1536,8 +1549,13 @@ class BonsaiAPI(object):
         )
 
         url = urljoin(self._api_url, url_path)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_STOP_LOGGING_ACTION, _BRAIN_VERSION_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_BRAIN_VERSION_OBJECT],
+        )
 
-        return self._post(url=url, debug=debug, output=output)
+        return self._post(url=url, debug=debug, output=output, event=event)
 
     def reset_training(
         self,
@@ -1565,7 +1583,13 @@ class BonsaiAPI(object):
         else:
             data = {"concepts": concepts}
 
-        return self._post(url=url, data=data, debug=debug, output=output)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_RESET_TRAINING_ACTION, _BRAIN_VERSION_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_BRAIN_VERSION_OBJECT],
+        )
+
+        return self._post(url=url, data=data, debug=debug, output=output, event=event)
 
     def start_assessment(
         self,
@@ -1848,7 +1872,11 @@ class BonsaiAPI(object):
             assessmentName=name,
         )
         url = urljoin(self._api_url, url_path)
-
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_START_ACTION, _ASSESSMENT_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_ASSESSMENT_OBJECT],
+        )
         data = {
             "name": name,
             "concept": concept_name,
@@ -1859,7 +1887,7 @@ class BonsaiAPI(object):
             "maximumDurationInMinutes": maximum_duration_in_minutes,
         }
 
-        return self._post(url=url, data=data, debug=debug, output=output)
+        return self._put(url=url, data=data, debug=debug, output=output, event=event)
 
     def list_assessment(
         self,
@@ -1880,8 +1908,13 @@ class BonsaiAPI(object):
             version=version,
         )
         url = urljoin(self._api_url, url_path)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_VIEW_ACTION, _ALL_ASSESSMENTS_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_BRAIN_VERSION_OBJECT],
+        )
 
-        return self._get(url=url, debug=debug, output=output)
+        return self._get(url=url, debug=debug, output=output, event=event)
 
     def get_assessment(
         self,
@@ -1905,8 +1938,13 @@ class BonsaiAPI(object):
             assessmentName=name,
         )
         url = urljoin(self._api_url, url_path)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_VIEW_ACTION, _ASSESSMENT_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_ASSESSMENT_OBJECT],
+        )
 
-        return self._get(url=url, debug=debug, output=output)
+        return self._get(url=url, debug=debug, output=output, event=event)
 
     def update_assessment(
         self,
@@ -1931,9 +1969,14 @@ class BonsaiAPI(object):
             assessmentName=name,
         )
         url = urljoin(self._api_url, url_path)
-
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_UPDATE_ACTION, _ASSESSMENT_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_ASSESSMENT_OBJECT],
+        )
         data = {"displayName": display_name, "description": description}
-        return self._put(url=url, data=data, debug=debug, output=output)
+
+        return self._patch(url=url, data=data, debug=debug, output=output, event=event)
 
     def stop_assessment_v2(
         self,
@@ -1957,9 +2000,14 @@ class BonsaiAPI(object):
             assessmentName=name,
         )
         url = urljoin(self._api_url, url_path)
-
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_STOP_ACTION, _ASSESSMENT_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_ASSESSMENT_OBJECT],
+        )
         data = {"state": state}
-        return self._put(url=url, data=data, debug=debug, output=output)
+
+        return self._patch(url=url, data=data, debug=debug, output=output, event=event)
 
     def delete_assessment(
         self,
@@ -1983,7 +2031,13 @@ class BonsaiAPI(object):
             assessmentName=name,
         )
         url = urljoin(self._api_url, url_path)
-        return self._delete(url=url, debug=debug, output=output)
+        event = self.application_insights_handler.create_event(
+            "{}{}".format(_DELETE_ACTION, _ASSESSMENT_OBJECT),
+            ObjectUri=[url_path],
+            ObjectType=[_ASSESSMENT_OBJECT],
+        )
+
+        return self._delete(url=url, debug=debug, output=output, event=event)
 
     def _raise_on_redirect(self, response: requests.Response):
         """Raises an HTTPError if the response is 301.
