@@ -4,6 +4,7 @@ This file contains the code for commands that target a bonsai brain in version 2
 __author__ = "Karthik Sankara Subramanian"
 __copyright__ = "Copyright 2020, Microsoft Corp."
 
+from typing import Any, Dict, List
 import click
 from json import dumps
 
@@ -12,10 +13,11 @@ from bonsai_cli.utils import (
     api,
     get_version_checker,
     raise_as_click_exception,
+    raise_204_click_exception,
     raise_brain_server_error_as_click_exception,
+    raise_client_side_click_exception,
     raise_unique_constraint_violation_as_click_exception,
     raise_not_found_as_click_exception,
-    raise_client_side_click_exception,
 )
 
 from .brain_version import version
@@ -77,6 +79,24 @@ def create_brain(
             output=output,
         )
 
+        status_message = "{} created.".format(response["name"])
+
+        if output == "json":
+            json_response = {
+                "status": response["status"],
+                "statusCode": response["statusCode"],
+                "statusMessage": status_message,
+            }
+
+            if test:
+                json_response["elapsed"] = str(response["elapsed"])
+                json_response["timeTaken"] = str(response["timeTaken"])
+
+            click.echo(dumps(json_response, indent=4))
+
+        else:
+            click.echo(status_message)
+
     except BrainServerError as e:
         if "Unique index constraint violation" in str(e):
             raise_unique_constraint_violation_as_click_exception(
@@ -88,23 +108,10 @@ def create_brain(
     except AuthenticationError as e:
         raise_as_click_exception(e)
 
-    status_message = "{} created.".format(response["name"])
-
-    if output == "json":
-        json_response = {
-            "status": response["status"],
-            "statusCode": response["statusCode"],
-            "statusMessage": status_message,
-        }
-
-        if test:
-            json_response["elapsed"] = str(response["elapsed"])
-            json_response["timeTaken"] = str(response["timeTaken"])
-
-        click.echo(dumps(json_response, indent=4))
-
-    else:
-        click.echo(status_message)
+    except Exception as e:
+        raise_client_side_click_exception(
+            output, test, "{}: {}".format(type(e), e.args)
+        )
 
     version_checker.check_cli_version(wait=True, print_up_to_date=False)
 
@@ -150,6 +157,31 @@ def show_brain(
             name, workspace=workspace_id, debug=debug, output=output
         )
 
+        if output == "json":
+            json_response = {
+                "name": response["name"],
+                "displayName": response["displayName"],
+                "description": response["description"],
+                "createdOn": response["createdTimeStamp"],
+                "modifiedOn": response["modifiedTimeStamp"],
+                "status": response["status"],
+                "statusCode": response["statusCode"],
+                "statusMessage": "",
+            }
+
+            if test:
+                json_response["elapsed"] = str(response["elapsed"])
+                json_response["timeTaken"] = str(response["timeTaken"])
+
+            click.echo(dumps(json_response, indent=4))
+
+        else:
+            click.echo("Name: {}".format(response["name"]))
+            click.echo("Display Name: {}".format(response["displayName"]))
+            click.echo("Description: {}".format(response["description"]))
+            click.echo("Created On: {}".format(response["createdTimeStamp"]))
+            click.echo("Modified On: {}".format(response["modifiedTimeStamp"]))
+
     except BrainServerError as e:
         if e.exception["statusCode"] == 404:
             raise_not_found_as_click_exception(
@@ -161,30 +193,10 @@ def show_brain(
     except AuthenticationError as e:
         raise_as_click_exception(e)
 
-    if output == "json":
-        json_response = {
-            "name": response["name"],
-            "displayName": response["displayName"],
-            "description": response["description"],
-            "createdOn": response["createdTimeStamp"],
-            "modifiedOn": response["modifiedTimeStamp"],
-            "status": response["status"],
-            "statusCode": response["statusCode"],
-            "statusMessage": "",
-        }
-
-        if test:
-            json_response["elapsed"] = str(response["elapsed"])
-            json_response["timeTaken"] = str(response["timeTaken"])
-
-        click.echo(dumps(json_response, indent=4))
-
-    else:
-        click.echo("Name: {}".format(response["name"]))
-        click.echo("Display Name: {}".format(response["displayName"]))
-        click.echo("Description: {}".format(response["description"]))
-        click.echo("Created On: {}".format(response["createdTimeStamp"]))
-        click.echo("Modified On: {}".format(response["modifiedTimeStamp"]))
+    except Exception as e:
+        raise_client_side_click_exception(
+            output, test, "{}: {}".format(type(e), e.args)
+        )
 
     version_checker.check_cli_version(wait=True, print_up_to_date=False)
 
@@ -239,6 +251,24 @@ def update_brain(
             output=output,
         )
 
+        status_message = "{} updated.".format(response["name"])
+
+        if output == "json":
+            json_response = {
+                "status": response["status"],
+                "statusCode": response["statusCode"],
+                "statusMessage": status_message,
+            }
+
+            if test:
+                json_response["elapsed"] = str(response["elapsed"])
+                json_response["timeTaken"] = str(response["timeTaken"])
+
+            click.echo(dumps(json_response, indent=4))
+
+        else:
+            click.echo(status_message)
+
     except BrainServerError as e:
         if e.exception["statusCode"] == 404:
             raise_not_found_as_click_exception(
@@ -250,23 +280,10 @@ def update_brain(
     except AuthenticationError as e:
         raise_as_click_exception(e)
 
-    status_message = "{} updated.".format(response["name"])
-
-    if output == "json":
-        json_response = {
-            "status": response["status"],
-            "statusCode": response["statusCode"],
-            "statusMessage": status_message,
-        }
-
-        if test:
-            json_response["elapsed"] = str(response["elapsed"])
-            json_response["timeTaken"] = str(response["timeTaken"])
-
-        click.echo(dumps(json_response, indent=4))
-
-    else:
-        click.echo(status_message)
+    except Exception as e:
+        raise_client_side_click_exception(
+            output, test, "{}: {}".format(type(e), e.args)
+        )
 
     version_checker.check_cli_version(wait=True, print_up_to_date=False)
 
@@ -303,37 +320,42 @@ def list_brain(
             workspace=workspace_id, debug=debug, output=output
         )
 
+        if len(response["value"]) == 0:
+            click.echo("No brains exist for the current user")
+            ctx.exit()
+
+        if output == "json":
+            dict_rows: List[Dict[str, Any]] = []
+            for brain in response["value"]:
+                dict_rows.append(brain["name"])
+
+            json_response = {
+                "value": dict_rows,
+                "status": response["status"],
+                "statusCode": response["statusCode"],
+                "statusMessage": "",
+            }
+
+            if test:
+                json_response["elapsed"] = str(response["elapsed"])
+                json_response["timeTaken"] = str(response["timeTaken"])
+
+            click.echo(dumps(json_response, indent=4))
+
+        else:
+            for brain in response["value"]:
+                click.echo(brain["name"])
+
     except BrainServerError as e:
         raise_brain_server_error_as_click_exception(debug, output, test, e)
 
     except AuthenticationError as e:
         raise_as_click_exception(e)
 
-    if len(response["value"]) == 0:
-        click.echo("No brains exist for the current user")
-        ctx.exit()
-
-    if output == "json":
-        dict_rows = []
-        for brain in response["value"]:
-            dict_rows.append(brain["name"])
-
-        json_response = {
-            "value": dict_rows,
-            "status": response["status"],
-            "statusCode": response["statusCode"],
-            "statusMessage": "",
-        }
-
-        if test:
-            json_response["elapsed"] = str(response["elapsed"])
-            json_response["timeTaken"] = str(response["timeTaken"])
-
-        click.echo(dumps(json_response, indent=4))
-
-    else:
-        for brain in response["value"]:
-            click.echo(brain["name"])
+    except Exception as e:
+        raise_client_side_click_exception(
+            output, test, "{}: {}".format(type(e), e.args)
+        )
 
     version_checker.check_cli_version(wait=True, print_up_to_date=False)
 
@@ -404,7 +426,7 @@ def delete_brain(
             )
 
             if response["statusCode"] == 204:
-                raise_client_side_click_exception(
+                raise_204_click_exception(
                     debug,
                     output,
                     test,
@@ -413,30 +435,30 @@ def delete_brain(
                     response,
                 )
 
+            status_message = "{} deleted.".format(name)
+
+            if output == "json":
+                json_response = {
+                    "status": response["status"],
+                    "statusCode": response["statusCode"],
+                    "statusMessage": status_message,
+                }
+
+                if test:
+                    json_response["elapsed"] = str(response["elapsed"])
+                    json_response["timeTaken"] = str(response["timeTaken"])
+
+                click.echo(dumps(json_response, indent=4))
+
+            else:
+                click.echo(status_message)
+
         except BrainServerError as e:
             raise_brain_server_error_as_click_exception(debug, output, test, e)
 
         except AuthenticationError as e:
             raise_as_click_exception(e)
 
-        status_message = "{} deleted.".format(name)
-
-        if output == "json":
-            json_response = {
-                "status": response["status"],
-                "statusCode": response["statusCode"],
-                "statusMessage": status_message,
-            }
-
-            if test:
-                json_response["elapsed"] = str(response["elapsed"])
-                json_response["timeTaken"] = str(response["timeTaken"])
-
-            click.echo(dumps(json_response, indent=4))
-
-        else:
-            click.echo(status_message)
-
     version_checker.check_cli_version(wait=True, print_up_to_date=False)
 
 
